﻿/// <reference path="_namespace.js" />
/// <reference path="Colour.js" />
/// <reference path="../CompareResult.js"/>
/// <reference path="GraphicsBase.js" />
/// <reference path="../DomElements.js" />
/// <reference path="_excanvas.js" />
/// <reference path="../math/Guid.js" />
/// <reference path="../geometry/shapes/Rectangle.js" />
$imports("ajax.geometry.shapes.Rectangle", "ajax.CompareResult");


// Copyright 2006 Google Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


// Known Issues:
//
// * Patterns are not implemented.
// * Radial gradient are not implemented. The VML version of these look very
//   different from the canvas one.
// * Clipping paths are not implemented.
// * Coordsize. The width and height attribute have higher priority than the
//   width and height style values which isn't correct.
// * Painting mode isn't implemented.
// * Canvas width/height should is using content-box by default. IE in
//   Quirks mode will draw the canvas using border-box. Either change your
//   doctype to HTML5
//   (http://www.whatwg.org/specs/web-apps/current-work/#the-doctype)
//   or use Box Sizing Behavior from WebFX
//   (http://webfx.eae.net/dhtml/boxsizing/boxsizing.html)
// * Non uniform scaling does not correctly scale strokes.
// * Optimize. There is always room for speed improvements.

if ($getDesignMode())
    CanvasRenderingContext2D = {};
    
// Only add this code if we do not already have a canvas implementation
if (!document.createElement("canvas").getContext)
{
    (function()
    {
        // alias some functions to make (compiled) code shorter
        var m = Math;
        var mr = m.round;
        var ms = m.sin;
        var mc = m.cos;
        var abs = m.abs;
        var sqrt = m.sqrt;

        // this is used for sub pixel precision
        var Z = 10;
        var Z2 = Z / 2;

        /**
        * This funtion is assigned to the <canvas> elements as element.getContext().
        * @this {HTMLElement}
        * @return {CanvasRenderingContext2D_}
        */
        function getContext()
        {
            return this.context_ ||
        (this.context_ = new CanvasRenderingContext2D_(this));
        }

        var slice = Array.prototype.slice;

        /**
        * Binds a function to an object. The returned function will always use the
        * passed in {@code obj} as {@code this}.
        *
        * Example:
        *
        *   g = bind(f, obj, a, b)
        *   g(c, d) // will do f.call(obj, a, b, c, d)
        *
        * @param {Function} f The function to bind the object to
        * @param {Object} obj The object that should act as this when the function
        *     is called
        * @param {*} var_args Rest arguments that will be used as the initial
        *     arguments when the function is called
        * @return {Function} A new function that has bound this
        */
        function bind(f, obj, var_args)
        {
            var a = slice.call(arguments, 2);
            return function()
            {
                return f.apply(obj, a.concat(slice.call(arguments)));
            };
        }

        var G_vmlCanvasManager_ = {
            init: function(opt_doc)
            {
                if (/MSIE/.test(navigator.userAgent) && !window.opera)
                {
                    var doc = opt_doc || document;
                    // Create a dummy element so that IE will allow canvas elements to be
                    // recognized.
                    doc.createElement('canvas');
                    doc.attachEvent('onreadystatechange', bind(this.init_, this, doc));
                }
            },

            init_: function(doc)
            {
                // create xmlns
                if (!doc.namespaces['g_vml_'])
                {
                    doc.namespaces.add('g_vml_', 'urn:schemas-microsoft-com:vml',
                           '#default#VML');

                }
                if (!doc.namespaces['g_o_'])
                {
                    doc.namespaces.add('g_o_', 'urn:schemas-microsoft-com:office:office',
                           '#default#VML');
                }

                // Setup default CSS.  Only add one style sheet per document
                if (!doc.styleSheets['ex_canvas_'])
                {
                    var ss = doc.createStyleSheet();
                    ss.owningElement.id = 'ex_canvas_';
                    ss.cssText = 'canvas{display:inline-block;overflow:hidden;' +
                    // default size is 300x150 in Gecko and Opera
            'text-align:left;width:300px;height:150px}' +
            'g_vml_\\:*{behavior:url(#default#VML)}' +
            'g_o_\\:*{behavior:url(#default#VML)}';

                }

                // find all canvas elements
                var els = doc.getElementsByTagName('canvas');
                for (var i = 0; i < els.length; i++)
                {
                    this.initElement(els[i]);
                }
            },

            /**
            * Public initializes a canvas element so that it can be used as canvas
            * element from now on. This is called automatically before the page is
            * loaded but if you are creating elements using createElement you need to
            * make sure this is called on the element.
            * @param {HTMLElement} el The canvas element to initialize.
            * @return {HTMLElement} the element that was created.
            */
            initElement: function(el)
            {
                if (!el.getContext)
                {

                    el.getContext = getContext;

                    // Remove fallback content. There is no way to hide text nodes so we
                    // just remove all childNodes. We could hide all elements and remove
                    // text nodes but who really cares about the fallback content.
                    el.innerHTML = '';

                    // do not use inline function because that will leak memory
                    el.attachEvent('onpropertychange', onPropertyChange);
                    el.attachEvent('onresize', onResize);

                    var attrs = el.attributes;
                    if (attrs.width && attrs.width.specified)
                    {
                        // TODO: use runtimeStyle and coordsize
                        // el.getContext().setWidth_(attrs.width.nodeValue);
                        el.style.width = attrs.width.nodeValue + 'px';
                    } else
                    {
                        el.width = el.clientWidth;
                    }
                    if (attrs.height && attrs.height.specified)
                    {
                        // TODO: use runtimeStyle and coordsize
                        // el.getContext().setHeight_(attrs.height.nodeValue);
                        el.style.height = attrs.height.nodeValue + 'px';
                    } else
                    {
                        el.height = el.clientHeight;
                    }
                    //el.getContext().setCoordsize_()
                }
                return el;
            }
        };

        function onPropertyChange(e)
        {
            var el = e.srcElement;

            switch (e.propertyName)
            {
                case 'width':
                    el.style.width = el.attributes.width.nodeValue + 'px';
                    el.getContext().clearRect();
                    break;
                case 'height':
                    el.style.height = el.attributes.height.nodeValue + 'px';
                    el.getContext().clearRect();
                    break;
            }
        }

        function onResize(e)
        {
            var el = e.srcElement;
            if (el.firstChild)
            {
                el.firstChild.style.width = el.clientWidth + 'px';
                el.firstChild.style.height = el.clientHeight + 'px';
            }
        }

        G_vmlCanvasManager_.init();

        // precompute "00" to "FF"
        var dec2hex = [];
        for (var i = 0; i < 16; i++)
        {
            for (var j = 0; j < 16; j++)
            {
                dec2hex[i * 16 + j] = i.toString(16) + j.toString(16);
            }
        }

        function createMatrixIdentity()
        {
            return [
      [1, 0, 0],
      [0, 1, 0],
      [0, 0, 1]
    ];
        }

        function matrixMultiply(m1, m2)
        {
            var result = createMatrixIdentity();

            for (var x = 0; x < 3; x++)
            {
                for (var y = 0; y < 3; y++)
                {
                    var sum = 0;

                    for (var z = 0; z < 3; z++)
                    {
                        sum += m1[x][z] * m2[z][y];
                    }

                    result[x][y] = sum;
                }
            }
            return result;
        }

        function copyState(o1, o2)
        {
            o2.fillStyle = o1.fillStyle;
            o2.lineCap = o1.lineCap;
            o2.lineJoin = o1.lineJoin;
            o2.lineWidth = o1.lineWidth;
            o2.miterLimit = o1.miterLimit;
            o2.shadowBlur = o1.shadowBlur;
            o2.shadowColor = o1.shadowColor;
            o2.shadowOffsetX = o1.shadowOffsetX;
            o2.shadowOffsetY = o1.shadowOffsetY;
            o2.strokeStyle = o1.strokeStyle;
            o2.globalAlpha = o1.globalAlpha;
            o2.arcScaleX_ = o1.arcScaleX_;
            o2.arcScaleY_ = o1.arcScaleY_;
            o2.lineScale_ = o1.lineScale_;
        }

        function processStyle(styleString)
        {
            var str, alpha = 1;

            styleString = String(styleString);
            if (styleString.substring(0, 3) == 'rgb')
            {
                var start = styleString.indexOf('(', 3);
                var end = styleString.indexOf(')', start + 1);
                var guts = styleString.substring(start + 1, end).split(',');

                str = '#';
                for (var i = 0; i < 3; i++)
                {
                    str += dec2hex[Number(guts[i])];
                }

                if (guts.length == 4 && styleString.substr(3, 1) == 'a')
                {
                    alpha = guts[3];
                }
            } else
            {
                str = styleString;
            }

            return { color: str, alpha: alpha };
        }

        function processLineCap(lineCap)
        {
            switch (lineCap)
            {
                case 'butt':
                    return 'flat';
                case 'round':
                    return 'round';
                case 'square':
                default:
                    return 'square';
            }
        }

        /**
        * This class implements CanvasRenderingContext2D interface as described by
        * the WHATWG.
        * @param {HTMLElement} surfaceElement The element that the 2D context should
        * be associated with
        */
        function CanvasRenderingContext2D_(surfaceElement)
        {
            this.m_ = createMatrixIdentity();

            this.mStack_ = [];
            this.aStack_ = [];
            this.currentPath_ = [];

            // Canvas context properties
            this.strokeStyle = '#000';
            this.fillStyle = '#000';

            this.lineWidth = 1;
            this.lineJoin = 'miter';
            this.lineCap = 'butt';
            this.miterLimit = Z * 1;
            this.globalAlpha = 1;
            this.canvas = surfaceElement;

            var el = surfaceElement.ownerDocument.createElement('div');
            el.style.width = surfaceElement.clientWidth + 'px';
            el.style.height = surfaceElement.clientHeight + 'px';
            el.style.overflow = 'hidden';
            el.style.position = 'absolute';
            surfaceElement.appendChild(el);

            this.element_ = el;
            this.arcScaleX_ = 1;
            this.arcScaleY_ = 1;
            this.lineScale_ = 1;
        }
        // Begin Added By Kevin Robinson
        CanvasRenderingContext2D_.prototype =
        {
            strokeStyle: "#000",
            fillStyle: "#000",
            lineWidth: 1,
            lineJoin: "miter",
            lineCap: "butt",
            miterLimit: Z * 1,
            globalAlpha: 1
        }
        // End Added
        var contextPrototype = CanvasRenderingContext2D_.prototype;
        contextPrototype.clearRect = function()
        {
            this.element_.innerHTML = '';
        };

        contextPrototype.beginPath = function()
        {
            // TODO: Branch current matrix so that save/restore has no effect
            //       as per safari docs.
            this.currentPath_ = [];
        };

        contextPrototype.moveTo = function(aX, aY)
        {
            var p = this.getCoords_(aX, aY);
            this.currentPath_.push({ type: 'moveTo', x: p.x, y: p.y });
            this.currentX_ = p.x;
            this.currentY_ = p.y;
        };

        contextPrototype.lineTo = function(aX, aY)
        {
            var p = this.getCoords_(aX, aY);
            this.currentPath_.push({ type: 'lineTo', x: p.x, y: p.y });

            this.currentX_ = p.x;
            this.currentY_ = p.y;
        };

        contextPrototype.bezierCurveTo = function(aCP1x, aCP1y,
                                            aCP2x, aCP2y,
                                            aX, aY)
        {
            var p = this.getCoords_(aX, aY);
            var cp1 = this.getCoords_(aCP1x, aCP1y);
            var cp2 = this.getCoords_(aCP2x, aCP2y);
            bezierCurveTo(this, cp1, cp2, p);
        };

        // Helper function that takes the already fixed cordinates.
        function bezierCurveTo(self, cp1, cp2, p)
        {
            self.currentPath_.push({
                type: 'bezierCurveTo',
                cp1x: cp1.x,
                cp1y: cp1.y,
                cp2x: cp2.x,
                cp2y: cp2.y,
                x: p.x,
                y: p.y
            });
            self.currentX_ = p.x;
            self.currentY_ = p.y;
        }

        contextPrototype.quadraticCurveTo = function(aCPx, aCPy, aX, aY)
        {
            // the following is lifted almost directly from
            // http://developer.mozilla.org/en/docs/Canvas_tutorial:Drawing_shapes

            var cp = this.getCoords_(aCPx, aCPy);
            var p = this.getCoords_(aX, aY);

            var cp1 = {
                x: this.currentX_ + 2.0 / 3.0 * (cp.x - this.currentX_),
                y: this.currentY_ + 2.0 / 3.0 * (cp.y - this.currentY_)
            };
            var cp2 = {
                x: cp1.x + (p.x - this.currentX_) / 3.0,
                y: cp1.y + (p.y - this.currentY_) / 3.0
            };

            bezierCurveTo(this, cp1, cp2, p);
        };

        contextPrototype.arc = function(aX, aY, aRadius,
                                  aStartAngle, aEndAngle, aClockwise)
        {
            aRadius *= Z;
            var arcType = aClockwise ? 'at' : 'wa';

            var xStart = aX + mc(aStartAngle) * aRadius - Z2;
            var yStart = aY + ms(aStartAngle) * aRadius - Z2;

            var xEnd = aX + mc(aEndAngle) * aRadius - Z2;
            var yEnd = aY + ms(aEndAngle) * aRadius - Z2;

            // IE won't render arches drawn counter clockwise if xStart == xEnd.
            if (xStart == xEnd && !aClockwise)
            {
                xStart += 0.125; // Offset xStart by 1/80 of a pixel. Use something
                // that can be represented in binary
            }

            var p = this.getCoords_(aX, aY);
            var pStart = this.getCoords_(xStart, yStart);
            var pEnd = this.getCoords_(xEnd, yEnd);

            this.currentPath_.push({ type: arcType,
                x: p.x,
                y: p.y,
                radius: aRadius,
                xStart: pStart.x,
                yStart: pStart.y,
                xEnd: pEnd.x,
                yEnd: pEnd.y
            });

        };

        contextPrototype.rect = function(aX, aY, aWidth, aHeight)
        {
            this.moveTo(aX, aY);
            this.lineTo(aX + aWidth, aY);
            this.lineTo(aX + aWidth, aY + aHeight);
            this.lineTo(aX, aY + aHeight);
            this.closePath();
        };

        contextPrototype.strokeRect = function(aX, aY, aWidth, aHeight)
        {
            var oldPath = this.currentPath_;
            this.beginPath();

            this.moveTo(aX, aY);
            this.lineTo(aX + aWidth, aY);
            this.lineTo(aX + aWidth, aY + aHeight);
            this.lineTo(aX, aY + aHeight);
            this.closePath();
            this.stroke();

            this.currentPath_ = oldPath;
        };

        contextPrototype.fillRect = function(aX, aY, aWidth, aHeight)
        {
            var oldPath = this.currentPath_;
            this.beginPath();

            this.moveTo(aX, aY);
            this.lineTo(aX + aWidth, aY);
            this.lineTo(aX + aWidth, aY + aHeight);
            this.lineTo(aX, aY + aHeight);
            this.closePath();
            this.fill();

            this.currentPath_ = oldPath;
        };

        contextPrototype.createLinearGradient = function(aX0, aY0, aX1, aY1)
        {///<returns type="CanvasGradient">Added by Kevin Robinson</returns>
            var gradient = new CanvasGradient_('gradient');
            gradient.x0_ = aX0;
            gradient.y0_ = aY0;
            gradient.x1_ = aX1;
            gradient.y1_ = aY1;
            return gradient;
        };

        contextPrototype.createRadialGradient = function(aX0, aY0, aR0,
                                                   aX1, aY1, aR1)
        {///<returns type="CanvasGradient">Added by Kevin Robinson</returns>
            var gradient = new CanvasGradient_('gradientradial');
            gradient.x0_ = aX0;
            gradient.y0_ = aY0;
            gradient.r0_ = aR0;
            gradient.x1_ = aX1;
            gradient.y1_ = aY1;
            gradient.r1_ = aR1;
            return gradient;
        };

        contextPrototype.drawImage = function(image, var_args)
        {
            var dx, dy, dw, dh, sx, sy, sw, sh;

            // to find the original width we overide the width and height
            var oldRuntimeWidth = image.runtimeStyle.width;
            var oldRuntimeHeight = image.runtimeStyle.height;
            image.runtimeStyle.width = 'auto';
            image.runtimeStyle.height = 'auto';

            // get the original size
            var w = image.width;
            var h = image.height;

            // and remove overides
            image.runtimeStyle.width = oldRuntimeWidth;
            image.runtimeStyle.height = oldRuntimeHeight;

            if (arguments.length == 3)
            {
                dx = arguments[1];
                dy = arguments[2];
                sx = sy = 0;
                sw = dw = w;
                sh = dh = h;
            } else if (arguments.length == 5)
            {
                dx = arguments[1];
                dy = arguments[2];
                dw = arguments[3];
                dh = arguments[4];
                sx = sy = 0;
                sw = w;
                sh = h;
            } else if (arguments.length == 9)
            {
                sx = arguments[1];
                sy = arguments[2];
                sw = arguments[3];
                sh = arguments[4];
                dx = arguments[5];
                dy = arguments[6];
                dw = arguments[7];
                dh = arguments[8];
            } else
            {
                throw Error('Invalid number of arguments');
            }

            var d = this.getCoords_(dx, dy);

            var w2 = sw / 2;
            var h2 = sh / 2;

            var vmlStr = [];

            var W = 10;
            var H = 10;

            // For some reason that I've now forgotten, using divs didn't work
            vmlStr.push(' <g_vml_:group',
                ' coordsize="', Z * W, ',', Z * H, '"',
                ' coordorigin="0,0"',
                ' style="width:', W, 'px;height:', H, 'px;position:absolute;');

            // If filters are necessary (rotation exists), create them
            // filters are bog-slow, so only create them if abbsolutely necessary
            // The following check doesn't account for skews (which don't exist
            // in the canvas spec (yet) anyway.

            if (this.m_[0][0] != 1 || this.m_[0][1])
            {
                var filter = [];

                // Note the 12/21 reversal
                filter.push('M11=', this.m_[0][0], ',',
                  'M12=', this.m_[1][0], ',',
                  'M21=', this.m_[0][1], ',',
                  'M22=', this.m_[1][1], ',',
                  'Dx=', mr(d.x / Z), ',',
                  'Dy=', mr(d.y / Z), '');

                // Bounding box calculation (need to minimize displayed area so that
                // filters don't waste time on unused pixels.
                var max = d;
                var c2 = this.getCoords_(dx + dw, dy);
                var c3 = this.getCoords_(dx, dy + dh);
                var c4 = this.getCoords_(dx + dw, dy + dh);

                max.x = m.max(max.x, c2.x, c3.x, c4.x);
                max.y = m.max(max.y, c2.y, c3.y, c4.y);

                vmlStr.push('padding:0 ', mr(max.x / Z), 'px ', mr(max.y / Z),
                  'px 0;filter:progid:DXImageTransform.Microsoft.Matrix(',
                  filter.join(''), ", sizingmethod='clip');")
            } else
            {
                vmlStr.push('top:', mr(d.y / Z), 'px;left:', mr(d.x / Z), 'px;');
            }

            vmlStr.push(' ">',
                '<g_vml_:image src="', image.src, '"',
                ' style="width:', Z * dw, 'px;',
                ' height:', Z * dh, 'px;"',
                ' cropleft="', sx / w, '"',
                ' croptop="', sy / h, '"',
                ' cropright="', (w - sx - sw) / w, '"',
                ' cropbottom="', (h - sy - sh) / h, '"',
                ' />',
                '</g_vml_:group>');

            this.element_.insertAdjacentHTML('BeforeEnd',
                                    vmlStr.join(''));
        };

        contextPrototype.stroke = function(aFill)
        {
            var lineStr = [];
            var lineOpen = false;
            var a = processStyle(aFill ? this.fillStyle : this.strokeStyle);
            var color = a.color;
            var opacity = a.alpha * this.globalAlpha;

            var W = 10;
            var H = 10;

            lineStr.push('<g_vml_:shape',
                 ' filled="', !!aFill, '"',
                 ' style="position:absolute;width:', W, 'px;height:', H, 'px;"',
                 ' coordorigin="0 0" coordsize="', Z * W, ' ', Z * H, '"',
                 ' stroked="', !aFill, '"',
                 ' path="');

            var newSeq = false;
            var min = { x: null, y: null };
            var max = { x: null, y: null };

            for (var i = 0; i < this.currentPath_.length; i++)
            {
                var p = this.currentPath_[i];
                var c;

                switch (p.type)
                {
                    case 'moveTo':
                        c = p;
                        lineStr.push(' m ', mr(p.x), ',', mr(p.y));
                        break;
                    case 'lineTo':
                        lineStr.push(' l ', mr(p.x), ',', mr(p.y));
                        break;
                    case 'close':
                        lineStr.push(' x ');
                        p = null;
                        break;
                    case 'bezierCurveTo':
                        lineStr.push(' c ',
                       mr(p.cp1x), ',', mr(p.cp1y), ',',
                       mr(p.cp2x), ',', mr(p.cp2y), ',',
                       mr(p.x), ',', mr(p.y));
                        break;
                    case 'at':
                    case 'wa':
                        lineStr.push(' ', p.type, ' ',
                       mr(p.x - this.arcScaleX_ * p.radius), ',',
                       mr(p.y - this.arcScaleY_ * p.radius), ' ',
                       mr(p.x + this.arcScaleX_ * p.radius), ',',
                       mr(p.y + this.arcScaleY_ * p.radius), ' ',
                       mr(p.xStart), ',', mr(p.yStart), ' ',
                       mr(p.xEnd), ',', mr(p.yEnd));
                        break;
                }


                // TODO: Following is broken for curves due to
                //       move to proper paths.

                // Figure out dimensions so we can do gradient fills
                // properly
                if (p)
                {
                    if (min.x == null || p.x < min.x)
                    {
                        min.x = p.x;
                    }
                    if (max.x == null || p.x > max.x)
                    {
                        max.x = p.x;
                    }
                    if (min.y == null || p.y < min.y)
                    {
                        min.y = p.y;
                    }
                    if (max.y == null || p.y > max.y)
                    {
                        max.y = p.y;
                    }
                }
            }
            lineStr.push(' ">');

            if (!aFill)
            {
                var lineWidth = this.lineScale_ * this.lineWidth;

                // VML cannot correctly render a line if the width is less than 1px.
                // In that case, we dilute the color to make the line look thinner.
                if (lineWidth < 1)
                {
                    opacity *= lineWidth;
                }

                lineStr.push(
        '<g_vml_:stroke',
        ' opacity="', opacity, '"',
        ' joinstyle="', this.lineJoin, '"',
        ' miterlimit="', this.miterLimit, '"',
        ' endcap="', processLineCap(this.lineCap), '"',
        ' weight="', lineWidth, 'px"',
        ' color="', color, '" />'
      );
            } else if (typeof this.fillStyle == 'object')
            {
                var fillStyle = this.fillStyle;
                var angle = 0;
                var focus = { x: 0, y: 0 };

                // additional offset
                var shift = 0;
                // scale factor for offset
                var expansion = 1;

                if (fillStyle.type_ == 'gradient')
                {
                    var x0 = fillStyle.x0_ / this.arcScaleX_;
                    var y0 = fillStyle.y0_ / this.arcScaleY_;
                    var x1 = fillStyle.x1_ / this.arcScaleX_;
                    var y1 = fillStyle.y1_ / this.arcScaleY_;
                    var p0 = this.getCoords_(x0, y0);
                    var p1 = this.getCoords_(x1, y1);
                    var dx = p1.x - p0.x;
                    var dy = p1.y - p0.y;
                    angle = Math.atan2(dx, dy) * 180 / Math.PI;

                    // The angle should be a non-negative number.
                    if (angle < 0)
                    {
                        angle += 360;
                    }

                    // Very small angles produce an unexpected result because they are
                    // converted to a scientific notation string.
                    if (angle < 1e-6)
                    {
                        angle = 0;
                    }
                } else
                {
                    var p0 = this.getCoords_(fillStyle.x0_, fillStyle.y0_);
                    var width = max.x - min.x;
                    var height = max.y - min.y;
                    focus = {
                        x: (p0.x - min.x) / width,
                        y: (p0.y - min.y) / height
                    };

                    width /= this.arcScaleX_ * Z;
                    height /= this.arcScaleY_ * Z;
                    var dimension = m.max(width, height);
                    shift = 2 * fillStyle.r0_ / dimension;
                    expansion = 2 * fillStyle.r1_ / dimension - shift;
                }

                // We need to sort the color stops in ascending order by offset,
                // otherwise IE won't interpret it correctly.
                var stops = fillStyle.colors_;
                stops.sort(function(cs1, cs2)
                {
                    return cs1.offset - cs2.offset;
                });

                var length = stops.length;
                var color1 = stops[0].color;
                var color2 = stops[length - 1].color;
                var opacity1 = stops[0].alpha * this.globalAlpha;
                var opacity2 = stops[length - 1].alpha * this.globalAlpha;

                var colors = [];
                for (var i = 0; i < length; i++)
                {
                    var stop = stops[i];
                    colors.push(stop.offset * expansion + shift + ' ' + stop.color);
                }

                // When colors attribute is used, the meanings of opacity and o:opacity2
                // are reversed.
                lineStr.push('<g_vml_:fill type="', fillStyle.type_, '"',
                   ' method="none" focus="100%"',
                   ' color="', color1, '"',
                   ' color2="', color2, '"',
                   ' colors="', colors.join(','), '"',
                   ' opacity="', opacity2, '"',
                   ' g_o_:opacity2="', opacity1, '"',
                   ' angle="', angle, '"',
                   ' focusposition="', focus.x, ',', focus.y, '" />');
            } else
            {
                lineStr.push('<g_vml_:fill color="', color, '" opacity="', opacity,
                   '" />');
            }

            lineStr.push('</g_vml_:shape>');

            this.element_.insertAdjacentHTML('beforeEnd', lineStr.join(''));
        };

        contextPrototype.fill = function()
        {
            this.stroke(true);
        }

        contextPrototype.closePath = function()
        {
            this.currentPath_.push({ type: 'close' });
        };

        /**
        * @private
        */
        contextPrototype.getCoords_ = function(aX, aY)
        {
            var m = this.m_;
            return {
                x: Z * (aX * m[0][0] + aY * m[1][0] + m[2][0]) - Z2,
                y: Z * (aX * m[0][1] + aY * m[1][1] + m[2][1]) - Z2
            }
        };

        contextPrototype.save = function()
        {
            var o = {};
            copyState(this, o);
            this.aStack_.push(o);
            this.mStack_.push(this.m_);
            this.m_ = matrixMultiply(createMatrixIdentity(), this.m_);
        };

        contextPrototype.restore = function()
        {
            copyState(this.aStack_.pop(), this);
            this.m_ = this.mStack_.pop();
        };

        function matrixIsFinite(m)
        {
            for (var j = 0; j < 3; j++)
            {
                for (var k = 0; k < 2; k++)
                {
                    if (!isFinite(m[j][k]) || isNaN(m[j][k]))
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        function setM(ctx, m, updateLineScale)
        {
            if (!matrixIsFinite(m))
            {
                return;
            }
            ctx.m_ = m;

            if (updateLineScale)
            {
                // Get the line scale.
                // Determinant of this.m_ means how much the area is enlarged by the
                // transformation. So its square root can be used as a scale factor
                // for width.
                var det = m[0][0] * m[1][1] - m[0][1] * m[1][0];
                ctx.lineScale_ = sqrt(abs(det));
            }
        }

        contextPrototype.translate = function(aX, aY)
        {
            var m1 = [
      [1, 0, 0],
      [0, 1, 0],
      [aX, aY, 1]
    ];

            setM(this, matrixMultiply(m1, this.m_), false);
        };

        contextPrototype.rotate = function(aRot)
        {
            var c = mc(aRot);
            var s = ms(aRot);

            var m1 = [
      [c, s, 0],
      [-s, c, 0],
      [0, 0, 1]
    ];

            setM(this, matrixMultiply(m1, this.m_), false);
        };

        contextPrototype.scale = function(aX, aY)
        {
            this.arcScaleX_ *= aX;
            this.arcScaleY_ *= aY;
            var m1 = [
      [aX, 0, 0],
      [0, aY, 0],
      [0, 0, 1]
    ];

            setM(this, matrixMultiply(m1, this.m_), true);
        };

        contextPrototype.transform = function(m11, m12, m21, m22, dx, dy)
        {
            var m1 = [
      [m11, m12, 0],
      [m21, m22, 0],
      [dx, dy, 1]
    ];

            setM(this, matrixMultiply(m1, this.m_), true);
        };

        contextPrototype.setTransform = function(m11, m12, m21, m22, dx, dy)
        {
            var m = [
      [m11, m12, 0],
      [m21, m22, 0],
      [dx, dy, 1]
    ];

            setM(this, m, true);
        };

        /******** STUBS ********/
        contextPrototype.clip = function()
        {
            // TODO: Implement
        };

        contextPrototype.arcTo = function()
        {
            // TODO: Implement
        };

        contextPrototype.createPattern = function()
        {
            return new CanvasPattern_;
        };

        // Gradient / Pattern Stubs
        function CanvasGradient_(aType)
        {
            this.type_ = aType;
            this.x0_ = 0;
            this.y0_ = 0;
            this.r0_ = 0;
            this.x1_ = 0;
            this.y1_ = 0;
            this.r1_ = 0;
            this.colors_ = [];
        }

        CanvasGradient_.prototype.addColorStop = function(aOffset, aColor)
        {
            aColor = processStyle(aColor);
            this.colors_.push({ offset: aOffset,
                color: aColor.color,
                alpha: aColor.alpha
            });
        };

        function CanvasPattern_() { }

        // set up externs
        G_vmlCanvasManager = G_vmlCanvasManager_;
        CanvasRenderingContext2D = CanvasRenderingContext2D_;
        CanvasGradient = CanvasGradient_;
        CanvasPattern = CanvasPattern_;

    })();

} // if no CanvasRenderingContext2D

    
// Begin Added by Kevin Robinson
CanvasRenderingContext2D.cast = function(obj)
{///<returns type="CanvasRenderingContext2D"/>
    return obj;
}
/* These two classes are not always defined on systems that have proper SVG graphics.
CanvasGradient.cast = function(obj)
{///<returns type="CanvasGradient"/>
    return obj;
}
CanvasPattern.cast = function(obj)
{///<returns type="CanvasPattern"/>
    return obj;
}
*/
// End Added




/*****************************************************************************
Ajax library class definition. Kevin Robinson 2006.
Class: ajax.drawing.GraphicsCanvas

Note: This class is merely a "wrapper" around the "Canvas" tag/IE VML Wrapper in order to
provide a consistant (as possible) interface between this and the std Graphics object.
/*****************************************************************************/

ajax.lang.addNamespace("ajax.drawing");

ajax.drawing.GraphicsCanvas = function(doc, parentEle, width, height)
{///<field name="canvas" type="domElement">The underlying canvas dom element (real or VML).</field>
    ///<field name="context" type="CanvasRenderingContext2D">The drawing surface.</field>
    ///<field name="contextFront" type="CanvasRenderingContext2D">The front buffer (although double buffering is not currently supported).</field>
    ///<field name="parentEle" type="domElement">The element containing the canvas.</field>
    ///<field name="doubleBuffering" type="boolean">Not supported.</field>
    ajax.drawing.GraphicsCanvas.superConstructor.call(this, doc, width, height);

    // If this is IE and VML has not run on this page yet, run it's initializer now.
//    if (ajax.Browser.isInternetExplorer && !doc.namespaces["g_vml_"])
//        G_vmlCanvasManager.init_(this.doc);

    // If this browser is IE and it cannot create a canvas and no vml namespace is yet created, create it here.
    if (ajax.Browser.isInternetExplorer && !document.createElement('canvas').getContext && !doc.namespaces["g_vml_"])
        G_vmlCanvasManager.init_(this.doc);

    this._initialize(parentEle, width, height);
}

ajax.lang.extend("ajax.drawing.GraphicsCanvas", "ajax.drawing.GraphicsBase");
if ($getDesignMode())
    ajax.drawing.GraphicsCanvas.prototype = ajax.drawing.GraphicsBase.prototype;

ajax.drawing.GraphicsCanvas.prototype._layer    = null;
ajax.drawing.GraphicsCanvas.prototype.canvas    = null;
ajax.drawing.GraphicsCanvas.prototype.context   = null;     // Back buffer
ajax.drawing.GraphicsCanvas.prototype.contextFront = null;  // Front buffer
ajax.drawing.GraphicsCanvas.prototype.parentEle = null;
ajax.drawing.GraphicsCanvas.prototype.doubleBuffering = false; // Not really implemented yet.


ajax.drawing.GraphicsCanvas.prototype.dispose = function()
{
    // Remove the node we created
    this.removeNode(this.canvas);
    ajax.drawing.GraphicsCanvas.superClass.dispose.call(this);
}



ajax.drawing.GraphicsCanvas.prototype.getCanvas = function()
{///<returns type="domElement"/>
    return this.canvas;
}
ajax.drawing.GraphicsCanvas.prototype.getContext = function()
{///<returns type="CanvasRenderingContext2D"/>
    return this.context;
}
ajax.drawing.GraphicsCanvas.prototype.getParentEle = function()
{///<returns type="domElement"/>
    return this._parentEle;
}


// Now go ahead and support the actual Interface
//-----------------------------------------------------------------------------

// Scale is always 1.
ajax.drawing.GraphicsCanvas.prototype.getScale = function(){ return 1; }

ajax.drawing.GraphicsCanvas.prototype.setColor        = function(colour)
{    
    //colour = ajax.drawing.Colour.cast(colour);       
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    // Add color method: .toRgbaString();
    //var sVal = "rgba(" + colour.r + "," + colour.g + "," + colour.b + "," + colour.a + ")";
    //var sVal = ["rgba(", colour.r, ",", colour.g, ",", colour.b, ",", colour.a, ")"].join("");
    var sVal = ajax.drawing.Colour.toRgbaString(colour);
    ctx.strokeStyle = sVal;
    ctx.fillStyle = sVal;
};

ajax.drawing.GraphicsCanvas.prototype.drawLine        = function(x1, y1, x2, y2)
{            
    var ctx = this.context;
    ctx.beginPath();
    ctx.moveTo(x1, y1);
    ctx.lineTo(x2, y2);
    ctx.closePath();
    ctx.stroke();
    
};

ajax.drawing.GraphicsCanvas.prototype.drawPolyline3   = function(x1, y1, x2, y2, x3, y3)
{
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    ctx.beginPath();
    ctx.moveTo(x1, y1);
    ctx.lineTo(x2, y2);
    ctx.lineTo(x3, y3);
    ctx.closePath();
    ctx.stroke();
    
    /*
    ctx.strokeStyle = "rgb(255, 255, 255)";
            ctx.fillStyle = "rgb(0, 255, 0)";
            ctx.beginPath();
            ctx.moveTo(10,10);
            ctx.lineTo(200,10);
            ctx.lineTo(200, 100);
            ctx.closePath();
            ctx.fill(); 
    */
    
};
ajax.drawing.GraphicsCanvas.prototype.drawRect        = function(x, y, width, height)
{
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    ctx.strokeRect(x, y, width, height);
};

ajax.drawing.GraphicsCanvas.prototype.fillRect        = function(x, y, width, height)
{
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    ctx.fillRect(x, y, width, height);
};

ajax.drawing.GraphicsCanvas.prototype.drawPolygon3    = function(x1, y1, x2, y2, x3, y3)
{            
    var ctx = this.context;// CanvasRenderingContext2D.cast(this.context);          
    ctx.beginPath();
    ctx.moveTo(x1, y1);
    ctx.lineTo(x2, y2);
    ctx.lineTo(x3, y3); 
    ctx.lineTo(x1, y1); 
    //ctx.closePath();      
    ctx.stroke();    
};

ajax.drawing.GraphicsCanvas.prototype.fillPolygon3    = function(x1, y1, x2, y2, x3, y3)
{    
    var ctx = this.context;// CanvasRenderingContext2D.cast(this.context);
    ctx.beginPath();
    ctx.moveTo(x1, y1);
    ctx.lineTo(x2, y2);
    ctx.lineTo(x3, y3);
    ctx.lineTo(x1, y1);
    //ctx.closePath();
    ctx.fill();
};

ajax.drawing.GraphicsCanvas.prototype.fillArc         = function(x, y, width, height, startAngle, arcAngle)
{
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    ctx.beginPath();
    ctx.arc(x, y, width, startAngle, arcAngle, false);
    ctx.fill();
};

ajax.drawing.GraphicsCanvas.prototype.drawEllipse     = function(x, y, width, height)
{
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    ctx.arc(x, y, width, 0, 2*Math.PI, false);    
    ctx.stroke();
};

ajax.drawing.GraphicsCanvas.prototype.fillEllipse     = function(x, y, width, height)
{
    this.fillArc(x, y, width, height, 0, 360);
    /*
    var ctx = CanvasRenderingContext2D.cast(this.context);
    ctx.arc(x, y, width, 0, 360, false);
    ctx.closePath();
    ctx.fill();
    */
};
ajax.drawing.GraphicsCanvas.prototype.setFont         = function(fontFamily, size, style)
{///<summary>Not officially supported</summary>
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    ctx.mozTextStyle = size + " " + fontFamily + " " + style;
};
ajax.drawing.GraphicsCanvas.prototype.drawString      = function(str, x, y)
{///<summary>Not officially supported</summary>
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    if (ctx.mozDrawText)
    {
        var ctx = CanvasRenderingContext2D.cast(this.context); 
        ctx.moveTo(x, y);
        //ctx.translate(x, y);
        ctx.mozDrawText(str);
    }
};
ajax.drawing.GraphicsCanvas.prototype.addImage        = function(strFullUrl){};
ajax.drawing.GraphicsCanvas.prototype.drawImage       = function(image, x, y, width, height)
{
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    var ctx = this.context;
    // Pass args in full (as I believe there are hidden params)
    ctx.drawImage.apply(ctx, arguments);
};

ajax.drawing.GraphicsCanvas.prototype.drawImageToRect = function(domImage, destinationRect)
{    
    var r = ajax.geometry.shapes.Rectangle.cast(destinationRect);
    if (r.width && r.height)
        this.drawImage(domImage, r.x, r.y, r.width, r.height);
    else
        this.drawImage(domImage, r.x, r.y);
}


ajax.drawing.GraphicsCanvas.prototype.drawImageFromRect = function(image, sx, sy, sWidth, sHeight, dx, dy, dWidth, dHeight)
{///<summary>Draws an image from a source rectangle</summary>
    var ctx = CanvasRenderingContext2D.cast(this.context);    
    // HACK: Seems that Safari does not include all overloads for drawImage, so use deprecated method for now.
    var method = (ajax.Browser.isSafari) ? ctx.drawImageFromRect : ctx.drawImage;    
    method.apply(ctx, arguments);    
}

ajax.drawing.GraphicsCanvas.prototype.clear           = function(colour)
{///<param name="colour" type="ajax.drawing.Colour"/>
    //var ctx = CanvasRenderingContext2D.cast(this.context);
    
    if (this.parentEle.ajaxdrawingClearColor)
    {
        if (colour.compareTo(this.parentEle.ajaxdrawingClearColor) != ajax.CompareResult.Equals) // It changed
        {
            // Change color of parentEle
            this.parentEle.style.backgroundColor = colour.toHex("#"); 
            // Store it
            this.parentEle.ajaxdrawingClearColor = colour;
        }
    }
    else // First time
    {
        // Change color of parentEle
        this.parentEle.style.backgroundColor = colour.toHex("#"); 
        // Store it
        this.parentEle.ajaxdrawingClearColor = colour;
    }
        
    
    
    var ctx = this.context;
    if (ajax.Browser.isOpera)
    {
        this.setColor(colour);
        ctx.fillRect(0, 0, this.width, this.height);
    }
    else
        ctx.clearRect(0, 0, this.width, this.height);
    
    // After clearing the canvas, setup a clip region (helps FF greatly)       
    ctx.beginPath();
    ctx.rect(0, 0, this.width, this.height);    
    ctx.clip();      
    ctx.closePath();  
    
};
ajax.drawing.GraphicsCanvas.prototype.repaint         = function()
{///<summary>If we are using a real canvas (not VML) and are using double buffering, then the buffers are swapped here.</summary> 
   
    if (this.doubleBuffering)
    {   
        var frontContext    = this.contextFront; // CanvasRenderingContext2D.cast(this.contextFront);    
        frontContext.drawImage(this.canvas, 0, 0, this.width, this.height);
    }
};
ajax.drawing.GraphicsCanvas.prototype.toString        = function(){ return "ajax.drawing.GraphicsCanvas"; };

ajax.drawing.GraphicsCanvas.prototype.hasLoaded       = function()
{
    return true;
}
ajax.drawing.GraphicsCanvas.prototype._initialize    = function(parentEle, width, height)
{       
    this.parentEle  = parentEle;
    //if (ajax.Browser.isOpera)
    
    //this.doubleBuffering = true;
    // Double buffering is not available is IE (as vml version cannot draw from another canvas).
    this.doubleBuffering = ajax.Browser.hasCanvas ? this.doubleBuffering : false;
    
    //this._layer     = this.getDocument().createElement("div");
    //this.parentEle.appendChild(this._layer);
    
    this.width      = width;
    this.height     = height;
    
    
    // Create front and back buffer canvas'
    this.canvasFront    = this.doc.createElement("canvas");    
    this.canvas         = this.doc.createElement("canvas");       
    
    
    //this.canvas.style.antialias = this.canvasFront.style.antialias = false; // Doesn't work!  
    
        
    // IE likes this:   
    this.canvas.style.width     = this.canvasFront.style.width  = this.width + "px";
    this.canvas.style.height    = this.canvasFront.style.height = this.height + "px";  
    // FF likes this:
    this.canvas.width           = this.canvasFront.width = this.width;
    this.canvas.height          = this.canvasFront.width = this.height;        
    //this.canvas.style.position  = this.canvasFront.style.position = "absolute";
    this.canvas.style.top       = this.canvasFront.style.top = "0px";
    this.canvas.style.left      = this.canvasFront.style.left = "0px";
    
    //this.canvas.globalCompositeOperation = this.canvasFront.globalCompositeOperation = "copy";
    
    // Double buffering is only available for CANVAS enabled browsers (which support drawing from another canvas)
    if (this.doubleBuffering)
    {    
        this.parentEle.appendChild(this.canvasFront); 
        //this.getDocument().body.appendChild(this.canvasFront);
        //this.canvas.style.display = "none";
        this.getDocument().body.appendChild(this.canvas);
                
        // IE code for if VML ever supported canvas drawing
//        if (ajax.Browser.isInternetExplorer)// typeof(G_vmlCanvasManager) != "undefined")     
//        {   
//            this.canvasFront = G_vmlCanvasManager.initElement(this.canvasFront); 
//            this.canvas      = G_vmlCanvasManager.initElement(this.canvas);
//        }
        
        if (this.canvasFront.getContext)
        {             
            this.context        = this.canvas.getContext("2d"); // FF
            this.contextFront   = this.canvasFront.getContext("2d");            
        }
        // IE code for if VML ever supported canvas drawing
//        else
//        {
//            this.context        = new CanvasRenderingContext2D(this.canvas); 
//            this.contextFront   = new CanvasRenderingContext2D(this.canvasFront);
//        }
    }
    else
    {
        this.parentEle.appendChild(this.canvas);   
        
        // If IE and no canvas, initailize VML engine.
        if (ajax.Browser.isInternetExplorer && !ajax.Browser.hasCanvas) // typeof(G_vmlCanvasManager) != "undefined")
            this.canvas = G_vmlCanvasManager.initElement(this.canvas); 
               
        if (this.canvas.getContext)        
            this.context        = this.canvas.getContext("2d"); //FF
        else
            this.context        = new CanvasRenderingContext2D(this.canvas); // IE without canvas.
    } 
    
    //this.context.globalAlpha = 0.3;
    //this.context.scale(0.5, 0.5);
    //this.context.miterLimit = 0;
    //this.context.rotate(90);
    //this.context.globalCompositeOperation = "copy";
    
     
}

ajax.drawing.GraphicsCanvas.cast = function(obj)
{///<returns type="ajax.drawing.GraphicsCanvas"/>
    return obj;
}